@@ -12,10 +12,12 @@ import weakref
 import warnings
 import threading
 import contextlib
+import json
+import typing as _tp
+from pprint import pformat
 
 from types import ModuleType
 from importlib import import_module
-from collections.abc import Mapping, Sequence
 import numpy as np
 
 from inspect import signature as pysignature # noqa: F401
@@ -27,6 +29,8 @@ from numba.core.config import (PYVERSION, MACHINE_BITS, # noqa: F401
 from numba.core import config
 from numba.core import types
 
+from collections.abc import Mapping, Sequence, MutableSet, MutableMapping
+
 
 def erase_traceback(exc_value):
     """
@@ -99,6 +103,11 @@ INPLACE_BINOPS_TO_OPERATORS = {
     '@=': operator.imatmul,
 }
 
+
+ALL_BINOPS_TO_OPERATORS = {**BINOPS_TO_OPERATORS,
+                           **INPLACE_BINOPS_TO_OPERATORS}
+
+
 UNARY_BUITINS_TO_OPERATORS = {
     '+': operator.pos,
     '-': operator.neg,
@@ -175,26 +184,12 @@ def shutting_down(globals=globals):
 # which atexit is True. Some of these finalizers may call shutting_down() to
 # check whether the interpreter is shutting down. For this to behave correctly,
 # we need to make sure that _at_shutdown is called before the finalizer exit
-# function. Since atexit operates as a LIFO stack, we first contruct a dummy
+# function. Since atexit operates as a LIFO stack, we first construct a dummy
 # finalizer then register atexit to ensure this ordering.
 weakref.finalize(lambda: None, lambda: None)
 atexit.register(_at_shutdown)
 
 
-def use_new_style_errors():
-    """Returns True if new style errors are to be used, false otherwise"""
-    # This uses `config` so as to make sure it gets the current value from the
-    # module as e.g. some tests mutate the config with `override_config`.
-    return config.CAPTURED_ERRORS == 'new_style'
-
-
-def use_old_style_errors():
-    """Returns True if old style errors are to be used, false otherwise"""
-    # This uses `config` so as to make sure it gets the current value from the
-    # module as e.g. some tests mutate the config with `override_config`.
-    return config.CAPTURED_ERRORS == 'old_style'
-
-
 class ThreadLocalStack:
     """A TLS stack container.
 
@@ -347,7 +342,62 @@ def order_by_target_specificity(target, templates, fnkey=''):
     return order
 
 
-class SortedMap(Mapping):
+T = _tp.TypeVar('T')
+
+
+class OrderedSet(MutableSet[T]):
+
+    def __init__(self, iterable: _tp.Iterable[T] = ()):
+        # Just uses a dictionary under-the-hood to maintain insertion order.
+        self._data = dict.fromkeys(iterable, None)
+
+    def __contains__(self, key):
+        return key in self._data
+
+    def __iter__(self):
+        return iter(self._data)
+
+    def __len__(self):
+        return len(self._data)
+
+    def add(self, item):
+        self._data[item] = None
+
+    def discard(self, item):
+        self._data.pop(item, None)
+
+
+class MutableSortedSet(MutableSet[T], _tp.Generic[T]):
+    """Mutable Sorted Set
+    """
+
+    def __init__(self, values: _tp.Iterable[T] = ()):
+        self._values = set(values)
+
+    def __len__(self):
+        return len(self._values)
+
+    def __iter__(self):
+        return iter(k for k in sorted(self._values))
+
+    def __contains__(self, x: T) -> bool:
+        return self._values.__contains__(x)
+
+    def add(self, x: T):
+        return self._values.add(x)
+
+    def discard(self, value: T):
+        self._values.discard(value)
+
+    def update(self, values):
+        self._values.update(values)
+
+
+Tk = _tp.TypeVar('Tk')
+Tv = _tp.TypeVar('Tv')
+
+
+class SortedMap(Mapping[Tk, Tv], _tp.Generic[Tk, Tv]):
     """Immutable
     """
 
@@ -369,6 +419,28 @@ class SortedMap(Mapping):
         return iter(k for k, v in self._values)
 
 
+class MutableSortedMap(MutableMapping[Tk, Tv], _tp.Generic[Tk, Tv]):
+    def __init__(self, dct=None):
+        if dct is None:
+            dct = {}
+        self._dct: dict[Tk, Tv] = dct
+
+    def __getitem__(self, k: Tk) -> Tv:
+        return self._dct[k]
+
+    def __setitem__(self, k: Tk, v: Tv):
+        self._dct[k] = v
+
+    def __delitem__(self, k: Tk):
+        del self._dct[k]
+
+    def __len__(self) -> int:
+        return len(self._dct)
+
+    def __iter__(self) -> int:
+        return iter(k for k in sorted(self._dct))
+
+
 class UniqueDict(dict):
     def __setitem__(self, key, value):
         if key in self:
@@ -376,69 +448,6 @@ class UniqueDict(dict):
         super(UniqueDict, self).__setitem__(key, value)
 
 
-if PYVERSION > (3, 7):
-    from functools import cached_property
-else:
-    from threading import RLock
-
-    # The following cached_property() implementation is adapted from CPython:
-    # https://github.com/python/cpython/blob/3.8/Lib/functools.py#L924-L976
-    # commit SHA: 12b714391e485d0150b343b114999bae4a0d34dd
-
-    ###########################################################################
-    ### cached_property() - computed once per instance, cached as attribute
-    ###########################################################################
-
-    _NOT_FOUND = object()
-
-    class cached_property:
-        def __init__(self, func):
-            self.func = func
-            self.attrname = None
-            self.__doc__ = func.__doc__
-            self.lock = RLock()
-
-        def __set_name__(self, owner, name):
-            if self.attrname is None:
-                self.attrname = name
-            elif name != self.attrname:
-                raise TypeError(
-                    "Cannot assign the same cached_property to two different names " # noqa: E501
-                    f"({self.attrname!r} and {name!r})."
-                )
-
-        def __get__(self, instance, owner=None):
-            if instance is None:
-                return self
-            if self.attrname is None:
-                raise TypeError(
-                    "Cannot use cached_property instance without calling __set_name__ on it.") # noqa: E501
-            try:
-                cache = instance.__dict__
-            except AttributeError:  # not all objects have __dict__ (e.g. class defines slots) # noqa: E501
-                msg = (
-                    f"No '__dict__' attribute on {type(instance).__name__!r} "
-                    f"instance to cache {self.attrname!r} property."
-                )
-                raise TypeError(msg) from None
-            val = cache.get(self.attrname, _NOT_FOUND)
-            if val is _NOT_FOUND:
-                with self.lock:
-                    # check if another thread filled cache while we awaited lock
-                    val = cache.get(self.attrname, _NOT_FOUND)
-                    if val is _NOT_FOUND:
-                        val = self.func(instance)
-                        try:
-                            cache[self.attrname] = val
-                        except TypeError:
-                            msg = (
-                                f"The '__dict__' attribute on {type(instance).__name__!r} instance "    # noqa: E501
-                                f"does not support item assignment for caching {self.attrname!r} property." # noqa: E501
-                            )
-                            raise TypeError(msg) from None
-            return val
-
-
 def runonce(fn):
     @functools.wraps(fn)
     def inner():
@@ -720,3 +729,55 @@ def get_hashable_key(value):
         return id(value)
     else:
         return value
+
+
+class threadsafe_cached_property(functools.cached_property):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._lock = threading.RLock()
+
+    def __get__(self, *args, **kwargs):
+        with self._lock:
+            return super().__get__(*args, **kwargs)
+
+
+def dump_llvm(fndesc, module):
+    print(("LLVM DUMP %s" % fndesc).center(80, '-'))
+    if config.HIGHLIGHT_DUMPS:
+        try:
+            from pygments import highlight
+            from pygments.lexers import LlvmLexer as lexer
+            from pygments.formatters import Terminal256Formatter
+            from numba.misc.dump_style import by_colorscheme
+            print(highlight(module.__repr__(), lexer(),
+                            Terminal256Formatter( style=by_colorscheme())))
+        except ImportError:
+            msg = "Please install pygments to see highlighted dumps"
+            raise ValueError(msg)
+    else:
+        print(module)
+    print('=' * 80)
+
+
+class _lazy_pformat(object):
+    """ Lazily generate strings that may be useful only for debugging.
+        pformat is the default formatter but you can pass lazy_func kwarg
+        to use a different formatter.
+    """
+    def __init__(self, *args, **kwargs):
+        self.func = pformat
+        self.args = args
+        self.kwargs = kwargs
+        if "lazy_func" in kwargs:
+            self.func = kwargs["lazy_func"]
+            del kwargs["lazy_func"]
+
+    def __str__(self):
+        return self.func(*self.args, **self.kwargs)
+
+
+class _LazyJSONEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, _lazy_pformat):
+            return str(obj)
+        return super().default(obj)
